@@ -3,6 +3,7 @@ from typing import Union
 
 import casadi as cs
 import numpy as np
+import numpy.typing as npt
 
 
 def array2cs(x: np.ndarray) -> Union[cs.SX, cs.MX]:
@@ -50,7 +51,7 @@ def array2cs(x: np.ndarray) -> Union[cs.SX, cs.MX]:
     return m
 
 
-def cs2array(x: Union[cs.MX, cs.SX]) -> np.ndarray:
+def cs2array(x: Union[cs.MX, cs.SX]) -> npt.NDArray[np.object_]:
     """Converts casadi symbolic variable `x` to a numpy array of scalars. Opposite to
     `array2cs`.
 
@@ -64,13 +65,13 @@ def cs2array(x: Union[cs.MX, cs.SX]) -> np.ndarray:
 
     Returns
     -------
-    np.ndarray
+    np array of cs.SX or MX
         The array containing the symbolic variable scalars.
     """
     if isinstance(x, np.ndarray):
         return x
     if isinstance(x, cs.DM):
-        return x.full()
+        return x.toarray()
 
     shape = x.shape
     y = np.empty(shape, object)
@@ -78,3 +79,35 @@ def cs2array(x: Union[cs.MX, cs.SX]) -> np.ndarray:
     for idx in indices:
         y[idx] = x[idx]
     return y
+
+
+def find_index_in_vector(
+    V: Union[cs.SX, cs.MX], a: Union[cs.SX, cs.MX]
+) -> npt.NDArray[np.int_]:
+    """Finds the indices of `a` in `V`.
+
+    Parameters
+    ----------
+    V : casadi.SX or MX
+        The vector in which to search.
+    a : casadi.SX or MX
+        The vector to search for.
+
+    Returns
+    -------
+    array of int
+        The indices of `a` in `V`.
+
+    Raises
+    ------
+    ValueError
+        Raises if `V` or `a` are not vectors.
+    """
+    if not V.is_vector() or not a.is_vector():
+        raise ValueError("`V` and `a` must be vectors.")
+
+    sp: cs.Sparsity = cs.jacobian_sparsity(a, V)
+    idx = np.asarray(sp.get_crs()[1], int)
+    if idx.size != a.numel():
+        raise RuntimeError("invalid subset: some entries of `a` were not found in `V`")
+    return idx
