@@ -1,27 +1,16 @@
 import warnings
+from collections.abc import Iterator, Sequence
 from itertools import count
-from typing import (
-    Any,
-    ClassVar,
-    Dict,
-    Iterator,
-    Literal,
-    Optional,
-    Sequence,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-)
+from typing import Any, ClassVar, Literal, Optional, TypeVar, Union
 
 import casadi as cs
 import numpy as np
 import numpy.typing as npt
 from joblib import Memory
 
-from csnlp.core.debug import NlpDebug
-from csnlp.nlps.objective import HasObjective
-from csnlp.util.io import SupportsDeepcopyAndPickle
+from ..core.debug import NlpDebug
+from ..util.io import SupportsDeepcopyAndPickle
+from .objective import HasObjective
 
 SymType = TypeVar("SymType", cs.SX, cs.MX)
 
@@ -36,7 +25,6 @@ class Nlp(HasObjective[SymType], SupportsDeepcopyAndPickle):
     constraints, objective, solver).
     """
 
-    __slots__ = ("id", "_debug")
     __ids: ClassVar[Iterator[int]] = count(0)
     is_multi: ClassVar[bool] = False
 
@@ -80,7 +68,7 @@ class Nlp(HasObjective[SymType], SupportsDeepcopyAndPickle):
         self._debug = NlpDebug() if debug else None
 
     @property
-    def sym_type(self) -> Type[SymType]:
+    def sym_type(self) -> type[SymType]:
         """Gets the CasADi symbolic type used in this NLP scheme."""
         return self._sym_type
 
@@ -98,7 +86,7 @@ class Nlp(HasObjective[SymType], SupportsDeepcopyAndPickle):
         """Gets whether the NLP instance is wrapped or not by the given wrapper type."""
         return False
 
-    def parameter(self, name: str, shape: Tuple[int, int] = (1, 1)) -> SymType:
+    def parameter(self, name: str, shape: tuple[int, int] = (1, 1)) -> SymType:
         out = super().parameter(name, shape)
         if self._debug is not None:
             self._debug.register("p", name, shape)
@@ -107,10 +95,10 @@ class Nlp(HasObjective[SymType], SupportsDeepcopyAndPickle):
     def variable(  # type: ignore[override]
         self,
         name: str,
-        shape: Tuple[int, int] = (1, 1),
+        shape: tuple[int, int] = (1, 1),
         lb: Union[npt.ArrayLike, cs.DM] = -np.inf,
         ub: Union[npt.ArrayLike, cs.DM] = +np.inf,
-    ) -> Tuple[SymType, SymType, SymType]:
+    ) -> tuple[SymType, SymType, SymType]:
         out = super().variable(name, shape, lb, ub)
         if self._debug is not None:
             self._debug.register("x", name, shape)
@@ -124,7 +112,7 @@ class Nlp(HasObjective[SymType], SupportsDeepcopyAndPickle):
         rhs: Union[SymType, np.ndarray, cs.DM],
         soft: bool = False,
         simplify: bool = True,
-    ) -> Tuple[SymType, ...]:
+    ) -> tuple[SymType, ...]:
         out = super().constraint(name, lhs, op, rhs, soft, simplify)
         if self._debug is not None:
             self._debug.register("g" if op == "==" else "h", name, out[0].shape)
@@ -137,7 +125,7 @@ class Nlp(HasObjective[SymType], SupportsDeepcopyAndPickle):
         outs: Sequence[SymType],
         name_in: Optional[Sequence[str]] = None,
         name_out: Optional[Sequence[str]] = None,
-        opts: Optional[Dict[Any, Any]] = None,
+        opts: Optional[dict[Any, Any]] = None,
     ) -> cs.Function:
         """Converts the optimization problem to an MX symbolic function. If the
         NLP is modelled in SX, the function will still be converted in MX since
@@ -170,10 +158,9 @@ class Nlp(HasObjective[SymType], SupportsDeepcopyAndPickle):
         Raises
         ------
         RuntimeError
-            Raises if the solver is uninitialized.
-        ValueError
-            Raises if the input or output expressions have free variables that
-            are not provided or cannot be computed by the solver.
+            Raises if the solver is uninitialized; or if the input or output expressions
+            have free variables that are not provided or cannot be computed by the
+            solver.
         """
         if self._sym_type is cs.SX:
             warnings.warn(
@@ -193,16 +180,6 @@ class Nlp(HasObjective[SymType], SupportsDeepcopyAndPickle):
             [self._p, self._x, self._lam_g, self._lam_h, self._lam_lbx, self._lam_ubx],
             outs,
         )
-        if Fin.has_free():
-            raise ValueError(
-                "Input expressions do not provide values for: "
-                f'{", ".join(Fin.get_free())}.'
-            )
-        if Fout.has_free():
-            raise ValueError(
-                "Output solver cannot provide values for: "
-                f'{", ".join(Fout.get_free())}.'
-            )
 
         # call the solver
         if self._sym_type is cs.SX:
@@ -214,8 +191,8 @@ class Nlp(HasObjective[SymType], SupportsDeepcopyAndPickle):
         sol = S(
             x0=x0,
             p=p,
-            lbx=self._lbx,
-            ubx=self._ubx,
+            lbx=self._lbx.data,
+            ubx=self._ubx.data,
             lbg=np.concatenate((np.zeros(self.ng), np.full(self.nh, -np.inf))),
             ubg=0,
             lam_x0=0,
@@ -224,8 +201,8 @@ class Nlp(HasObjective[SymType], SupportsDeepcopyAndPickle):
         x = sol["x"]
         lam_g = sol["lam_g"][: self.ng, :]
         lam_h = sol["lam_g"][self.ng :, :]
-        lam_lbx = -cs.fmin(sol["lam_x"], 0)
-        lam_ubx = cs.fmax(sol["lam_x"], 0)
+        lam_lbx = -cs.fmin(sol["lam_x"], 0)[self.nonmasked_lbx_idx, :]
+        lam_ubx = cs.fmax(sol["lam_x"], 0)[self.nonmasked_ubx_idx, :]
         Fsol = cs.Function("Fsol", ins, [x, lam_g, lam_h, lam_lbx, lam_ubx])
 
         # build final function
