@@ -1,24 +1,10 @@
 import pickle
-from copy import _reconstruct, deepcopy  # type: ignore[attr-defined]
-from functools import cached_property
-from inspect import getmembers
-from itertools import chain
+from copy import _reconstruct, deepcopy
 from os.path import splitext
 from pickletools import optimize
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    Dict,
-    Iterable,
-    List,
-    Literal,
-    Optional,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-)
+from typing import TYPE_CHECKING, Any, Callable, Literal, Optional, TypeVar
+
+from ..core.cache import invalidate_caches_of
 
 if TYPE_CHECKING:
     from scipy.io.matlab import mat_struct
@@ -65,26 +51,6 @@ def is_pickleable(obj: Any) -> bool:
         return False
 
 
-def _get_dict_state(
-    obj: "SupportsDeepcopyAndPickle",
-    attributes: Iterable[str],
-    fullstate: bool,
-    remove_None: bool,
-) -> Dict[str, Any]:
-    """Internal utility for SupportsDeepcopyAndPickle."""
-    state: Dict[str, Any] = {attr: getattr(obj, attr, None) for attr in attributes}
-    if not fullstate:
-        for attr in list(state.keys()):
-            val = state[attr]
-            if (
-                (remove_None and val is None)
-                or is_casadi_object(val)
-                or not is_pickleable(val)
-            ):
-                state.pop(attr, None)
-    return state
-
-
 T = TypeVar("T", bound="SupportsDeepcopyAndPickle")
 
 
@@ -112,16 +78,10 @@ class SupportsDeepcopyAndPickle:
         """
         new = deepcopy(self)
         if invalidate_caches:
-            # basically do again what csnlp.util.funcs.invalidate_cache does
-            for membername, member in getmembers(type(new)):
-                if hasattr(member, "cache_clear"):
-                    getattr(new, membername).cache_clear()
-                elif isinstance(member, cached_property):
-                    if membername in new.__dict__:
-                        del new.__dict__[membername]
+            invalidate_caches_of(new)
         return new
 
-    def __deepcopy__(self: T, memo: Optional[Dict[int, List[Any]]] = None) -> T:
+    def __deepcopy__(self: T, memo: Optional[dict[int, list[Any]]] = None) -> T:
         """Returns a deepcopy of the object."""
         rv = self.__reduce_ex__(4)
         if isinstance(rv, str):
@@ -132,27 +92,22 @@ class SupportsDeepcopyAndPickle:
         new_rv = (*rv[:2], fullstate, *rv[3:])
         return _reconstruct(self, memo, *new_rv)
 
-    def __getstate__(
-        self: T,
-        fullstate: bool = False,
-    ) -> Union[None, Dict[str, Any], Tuple[Optional[Dict[str, Any]], Dict[str, Any]]]:
+    def __getstate__(self: T, fullstate: bool = False) -> Optional[dict[str, Any]]:
         """Returns the instance's state to be pickled/deepcopied."""
         # https://docs.python.org/3/library/pickle.html#pickle-inst
-        dictstate = (
-            _get_dict_state(self, self.__dict__.keys(), fullstate, False)
-            if hasattr(self, "__dict__") and self.__dict__.keys()
-            else None
-        )
-        slots = list(
-            chain.from_iterable(
-                getattr(cls, "__slots__", []) for cls in type(self).__mro__
-            )
-        )
-        slotstate = _get_dict_state(self, slots, fullstate, True) if slots else None
-        return (dictstate, slotstate) if slotstate is not None else dictstate
+        if not (hasattr(self, "__dict__") and self.__dict__.keys()):
+            return None
+        state = self.__dict__.copy()
+        if fullstate:
+            return state
+        state_items_copy = list(state.items())
+        for attr, val in state_items_copy:
+            if is_casadi_object(val) or not is_pickleable(val):
+                state.pop(attr, None)
+        return state
 
 
-_COMPRESSION_EXTS: Dict[str, Optional[str]] = {
+_COMPRESSION_EXTS: dict[str, Optional[str]] = {
     ".pkl": None,
     ".xz": "lzma",
     ".pbz2": "bz2",
@@ -201,9 +156,7 @@ def save(
 
     actual_ext = splitext(filename)[1]
     if compression is None:
-        compression = _COMPRESSION_EXTS.get(  # type: ignore[assignment]
-            actual_ext, None
-        )
+        compression = _COMPRESSION_EXTS.get(actual_ext)
 
     open_fun: Callable
     compress_fun: Callable
@@ -242,7 +195,6 @@ def save(
         open_fun = open
         compress_fun = blosc2.compress
     elif compression == "matlab":
-
         expected_ext = ".mat"
     else:
         raise ValueError(f"Unknown compression method {compression}.")
@@ -266,7 +218,7 @@ def save(
     return filename
 
 
-def load(filename: str) -> Dict[str, Any]:
+def load(filename: str) -> dict[str, Any]:
     """Loads data from a (possibly compressed) file.
 
     Parameters
@@ -343,11 +295,11 @@ def load(filename: str) -> Dict[str, Any]:
     return data
 
 
-def _check_mat_keys(dictionary: Dict, mat_struct_type: Type) -> Dict:
+def _check_mat_keys(dictionary: dict, mat_struct_type: type) -> dict:
     """Internal utility to check if entries in dictionary are mat-objects. If yes,
     todict is called to change them to nested dictionaries."""
 
-    def _todict_recursive(matobj: "mat_struct") -> Dict:
+    def _todict_recursive(matobj: "mat_struct") -> dict:
         dictionary = {}
         for strg in matobj._fieldnames:
             elem = matobj.__dict__[strg]
