@@ -1,14 +1,14 @@
-from typing import Callable, Dict, Literal, Optional, Tuple, TypeVar, Union
+from typing import Callable, Literal, Optional, TypeVar, Union
 
 import casadi as cs
 import numpy as np
 import numpy.typing as npt
 
-from csnlp.core.cache import cached_property, invalidate_cache
-from csnlp.core.data import array2cs, cs2array
-from csnlp.core.derivatives import hohessian, hojacobian
-from csnlp.core.solutions import Solution
-from csnlp.wrappers.wrapper import Nlp, Wrapper
+from ..core.cache import cached_property, invalidate_cache
+from ..core.data import array2cs, cs2array, find_index_in_vector
+from ..core.derivatives import hohessian, hojacobian
+from ..core.solutions import Solution
+from .wrapper import Nlp, Wrapper
 
 SymType = TypeVar("SymType", cs.SX, cs.MX)
 
@@ -16,7 +16,10 @@ SymType = TypeVar("SymType", cs.SX, cs.MX)
 class NlpSensitivity(Wrapper[SymType]):
     """
     Wraps an NLP to allow to perform numerical sensitivity analysis and compute its
-    derivates. See [1] for nonlinear programming sensitivity analysis.
+    derivates. See [1] for nonlinear programming sensitivity analysis. The computations
+    are tailored to IPOPT as solver, so no guarantees are provided for other solvers.
+    This is due to the fact that IPOPT, as an interior-point solver, guarantees that the
+    KKT conditions at the solution are satisfied. This may not the case for others.
 
     References
     ----------
@@ -26,8 +29,6 @@ class NlpSensitivity(Wrapper[SymType]):
         Springer, Berlin, Heidelberg.
     """
 
-    __slots__ = ("include_barrier_term", "_p_idx_internal", "_tau")
-
     def __init__(
         self,
         nlp: Nlp[SymType],
@@ -56,7 +57,7 @@ class NlpSensitivity(Wrapper[SymType]):
         super().__init__(nlp)
         self.include_barrier_term: bool = include_barrier_term
         self.set_target_parameters(target_parameters)
-        self._tau = nlp.sym_type.sym("tau") if include_barrier_term else 0
+        self._tau = nlp.sym_type.sym("tau") if include_barrier_term else None
 
     @property
     def target_parameters(self) -> SymType:
@@ -67,18 +68,16 @@ class NlpSensitivity(Wrapper[SymType]):
     @cached_property
     def lagrangian(self) -> SymType:
         """Gets the Lagrangian of the NLP problem (usually, `L`)."""
-        h_lbx, lam_h_lbx = self.nlp.h_lbx
-        h_ubx, lam_h_ubx = self.nlp.h_ubx
         return (
             self.nlp.f
             + cs.dot(self.nlp.lam_g, self.nlp.g)
             + cs.dot(self.nlp.lam_h, self.nlp.h)
-            + cs.dot(lam_h_lbx, h_lbx)
-            + cs.dot(lam_h_ubx, h_ubx)
+            + cs.dot(self.nlp.lam_lbx, self.nlp.h_lbx)
+            + cs.dot(self.nlp.lam_ubx, self.nlp.h_ubx)
         )
 
     @cached_property
-    def kkt(self) -> Tuple[SymType, Optional[SymType]]:
+    def kkt(self) -> tuple[SymType, Optional[SymType]]:
         """Gets the KKT conditions of the NLP problem in vector form, i.e.,
         ```
                             |      dLdx     |
@@ -95,26 +94,19 @@ class NlpSensitivity(Wrapper[SymType]):
                             diag(lam_h)*H + tau
         ```
         which is also returned as the second element of the tuple. Otherwise, `tau` is
-        `None`.
-
-        Note: The order of the KKT conditions can be adjusted via the class attribute
-        `_PRIMAL_DUAL_ORDER`.
-        """
-        items = {
-            "g": self.nlp.g,
-            "h": (self.nlp.lam_h * self.nlp.h) + self._tau,
-            "h_lbx": (self.nlp.h_lbx[0] * self.nlp.h_lbx[1]) + self._tau,
-            "h_ubx": (self.nlp.h_ubx[0] * self.nlp.h_ubx[1]) + self._tau,
-        }
+        `None`."""
+        tau = self._tau if self._tau is not None else 0
         kkt = cs.vertcat(
             cs.jacobian(self.lagrangian, self.nlp.x).T,
-            *(items.pop(v) for v in self.nlp.dual_variables_order)
+            self.nlp.g,
+            (self.nlp.lam_h * self.nlp.h) + tau,
+            (self.nlp.h_lbx * self.nlp.lam_lbx) + tau,
+            (self.nlp.h_ubx * self.nlp.lam_ubx) + tau,
         )
-        assert not items, "Internal error. `dual_variables_order` modified."
-        return kkt, (self._tau if self.include_barrier_term else None)
+        return kkt, self._tau
 
     @cached_property
-    def jacobians(self) -> Dict[str, SymType]:
+    def jacobians(self) -> dict[str, SymType]:
         """Computes various partial derivatives, which are then grouped in a dict with
         the following entries
             - `L-x`: lagrangian w.r.t. primal variables
@@ -127,7 +119,7 @@ class NlpSensitivity(Wrapper[SymType]):
         L = self.lagrangian
         K = self.kkt[0]
         x = self.nlp.x
-        y, y_idx = self._y_idx
+        y = self.nlp.primal_dual
         p, p_idx = self._p_idx
         return {
             "L-x": cs.jacobian(L, x).T,
@@ -135,11 +127,11 @@ class NlpSensitivity(Wrapper[SymType]):
             "g-x": cs.jacobian(self.nlp.g, x),
             "h-x": cs.jacobian(self.nlp.h, x),
             "K-p": cs.jacobian(K, p)[:, p_idx],
-            "K-y": cs.jacobian(K, y)[:, y_idx],
+            "K-y": cs.jacobian(K, y),
         }
 
     @cached_property
-    def hessians(self) -> Dict[str, SymType]:
+    def hessians(self) -> dict[str, SymType]:
         """Computes various partial hessians, which are then grouped in a dict with the
         following entries
             - `L-pp`: lagrangian w.r.t. parameters (twice)
@@ -157,7 +149,7 @@ class NlpSensitivity(Wrapper[SymType]):
         }
 
     @cached_property
-    def hojacobians(self) -> Dict[str, np.ndarray]:
+    def hojacobians(self) -> dict[str, np.ndarray]:
         """Computes various 3D jacobians, which are then grouped in a dict with the
         following entries
             - `K-pp`: kkt conditions w.r.t. parameters (twice)
@@ -168,44 +160,43 @@ class NlpSensitivity(Wrapper[SymType]):
         jacobians = self.jacobians
         Kp = jacobians["K-p"]
         Ky = jacobians["K-y"]
-        y, y_idx = self._y_idx
+        y = self.nlp.primal_dual
         p, p_idx = self._p_idx
         return {
             "K-pp": hojacobian(Kp, p)[..., p_idx, 0],
             "K-yp": hojacobian(Ky, p)[..., p_idx, 0],
-            "K-yy": hojacobian(Ky, y)[..., y_idx, 0],
-            "K-py": hojacobian(Kp, y)[..., y_idx, 0],
+            "K-yy": hojacobian(Ky, y)[..., 0],
+            "K-py": hojacobian(Kp, y)[..., 0],
         }
 
     @property
     def licq(self) -> SymType:
         """Gets the symbolic matrix for LICQ, defined as
         ```
-                    LICQ = [ dgdx^T, dhdx^T ]^T
+                    LICQ = [ dg dx, dh dx, dh_lbx dx, dh_ubx dx ]
         ```
         If the matrix is linear independent, then the NLP satisfies the Linear
         Independence Constraint Qualification.
 
-        Note:
-            1) the LICQ are computed for only the active inenquality constraints. Since
-               this is a symbolic representation, all are included, and it's up to the
-               user to eliminate the inactive.
-
-            2) lower and upper bound inequality constraints are not included in `h`
-               since they are by nature linear independent.
+        Note
+        -----
+        The LICQ is computed for only the active inenquality constraints. Since this is
+        a symbolic representation, all constraints are included, and it's up to the user
+        to eliminate the inactive ones.
         """
-        return cs.vertcat(self.jacobians["g-x"], self.jacobians["h-x"])
+        return cs.vertcat(
+            self.jacobians["g-x"],
+            self.jacobians["h-x"],
+            cs.jacobian(self.nlp.h_lbx, self.nlp.x),
+            cs.jacobian(self.nlp.h_ubx, self.nlp.x),
+        )
 
     def parametric_sensitivity(
         self,
         expr: SymType = None,
         solution: Optional[Solution[SymType]] = None,
         second_order: bool = False,
-    ) -> Union[
-        Tuple[SymType, Optional[SymType]],
-        Tuple[cs.DM, Optional[cs.DM]],
-        Tuple[npt.NDArray[np.double], Optional[npt.NDArray[np.double]]],
-    ]:
+    ) -> Union[tuple[SymType, Optional[SymType]], tuple[cs.DM, Optional[cs.DM]]]:
         """Performs the (symbolic or numerical) sensitivity of the NLP w.r.t. its
         parametrization, according to [1].
 
@@ -226,7 +217,7 @@ class NlpSensitivity(Wrapper[SymType]):
 
         Returns
         -------
-        2-element tuple of casadi.SX, or MX, or DM, or arrays
+        2-element tuple of casadi.SX, or MX, or DM
             The 1st and 2nd-order NLP parametric sensitivity in the form of an array/DM,
             if a solution is passed, or a symbolic vector SX/MX. When
             `second_order=False` the second element in the tuple is `None`.
@@ -258,7 +249,7 @@ class NlpSensitivity(Wrapper[SymType]):
         if Z is not None:
             Zshape = Z.shape
             Z = cs.vec(Z)
-            y, y_idx = self._y_idx
+            y = self.nlp.primal_dual
             p, p_idx = self._p_idx
             np_ = p.shape[0] if isinstance(p_idx, slice) else p_idx.size
 
@@ -269,7 +260,7 @@ class NlpSensitivity(Wrapper[SymType]):
                 Zp = hojacobian(Z, p)
                 Zy = hojacobian(Z, y)
             Zp = d(array2cs(Zp[:, 0, p_idx, 0]))
-            Zy = d(array2cs(Zy[:, 0, y_idx, 0]))
+            Zy = d(array2cs(Zy[:, 0, :, 0]))
 
             dZdp = Zy @ dydp + Zp
             dZdp = cs2array(dZdp).reshape(Zshape + (np_,), order="F").squeeze()
@@ -288,10 +279,10 @@ class NlpSensitivity(Wrapper[SymType]):
             return dydp, d2ydp2
 
         # second order sensitivity of Z, a.k.a., d2Zdp2
-        Zyp = d(hohessian(Z, y, p)[0][:, 0, y_idx, 0][:, :, p_idx, 0])
-        Zpy = d(hohessian(Z, p, y)[0][:, 0, p_idx, 0][:, :, y_idx, 0])
+        Zyp = d(hohessian(Z, y, p)[0][:, 0, :, 0][:, :, p_idx, 0])
+        Zpy = d(hohessian(Z, p, y)[0][:, 0, p_idx, 0][:, :, :, 0])
         Zpp = d(Zpp[:, 0, p_idx, 0][:, :, p_idx, 0])
-        Zyy = d(Zyy[:, 0, y_idx, 0][:, :, y_idx, 0])
+        Zyy = d(Zyy[:, 0, :, 0][:, :, :, 0])
         T1 = (d2ydp2.transpose((1, 2, 0)) @ cs2array(Zy).T).transpose((2, 0, 1))
         T2 = Zyy.transpose((0, 2, 1)) @ dydp_np + Zpy.transpose((0, 2, 1)) + Zyp
         d2Zdp2 = Zpp + T1 + dydp_np.T @ T2
@@ -305,11 +296,7 @@ class NlpSensitivity(Wrapper[SymType]):
         solution: Optional[Solution[SymType]],
         second_order: bool,
         d: Callable[[SymType], Union[SymType, cs.DM]],
-    ) -> Union[
-        Tuple[SymType, np.ndarray, SymType],
-        Tuple[cs.DM, np.ndarray, cs.DM],
-        Tuple[npt.NDArray[np.double], np.ndarray, npt.NDArray[np.double]],
-    ]:
+    ) -> Union[tuple[SymType, np.ndarray, SymType], tuple[cs.DM, np.ndarray, cs.DM]]:
         """Internal utility to compute the sensitivity of y w.r.t. p."""
         # first order sensitivity, a.k.a., dydp
         Ky = d(self.jacobians["K-y"])
@@ -336,7 +323,7 @@ class NlpSensitivity(Wrapper[SymType]):
         return dydp, dydp_, d2ydp2
 
     @invalidate_cache(jacobians, hessians, hojacobians)
-    def parameter(self, name: str, shape: Tuple[int, int] = (1, 1)) -> SymType:
+    def parameter(self, name: str, shape: tuple[int, int] = (1, 1)) -> SymType:
         """See `Nlp.parameter` method."""
         return self.nlp.parameter(name, shape)
 
@@ -344,10 +331,10 @@ class NlpSensitivity(Wrapper[SymType]):
     def variable(
         self,
         name: str,
-        shape: Tuple[int, int] = (1, 1),
+        shape: tuple[int, int] = (1, 1),
         lb: Union[npt.ArrayLike, cs.DM] = -np.inf,
         ub: Union[npt.ArrayLike, cs.DM] = +np.inf,
-    ) -> Tuple[SymType, SymType, SymType]:
+    ) -> tuple[SymType, SymType, SymType]:
         """See `Nlp.variable` method."""
         return self.nlp.variable(name, shape, lb, ub)
 
@@ -360,7 +347,7 @@ class NlpSensitivity(Wrapper[SymType]):
         rhs: Union[SymType, np.ndarray, cs.DM],
         soft: bool = False,
         simplify: bool = True,
-    ) -> Tuple[SymType, ...]:
+    ) -> tuple[SymType, ...]:
         """See `Nlp.constraint` method."""
         return self.nlp.constraint(name, lhs, op, rhs, soft, simplify)
 
@@ -376,49 +363,23 @@ class NlpSensitivity(Wrapper[SymType]):
         Parameters
         ----------
         parameters : casadi.MX or SX or None
-            New parameters to target during the sensitivity analyses. If `None`, all
-            NLP parameters are included.
+            New parameters to target during the sensitivity analyses. If `None`, all NLP
+            parameters are included.
         """
         if parameters is None:
             self._p_idx_internal = None, slice(None)
             return
-
         p: SymType = cs.vec(parameters)
         if self.nlp.sym_type is cs.SX:
             self._p_idx_internal = p, slice(None)
             return
-
-        sp_J: cs.Sparsity = cs.jacobian(p, self.nlp.p).sparsity()
-        idx = np.asarray(sp_J.get_crs()[1], int)
-        assert idx.size == p.shape[0], (
-            "Invalid subset of target parameters (some were not found in the"
-            " original NLP parameters)."
-        )
-        self._p_idx_internal = self.nlp.p, idx  # type: ignore[assignment]
+        self._p_idx_internal = self.nlp.p, find_index_in_vector(self.nlp.p, p)
 
     @property
-    def _y_idx(self) -> Tuple[SymType, Union[slice, npt.NDArray[np.int64]]]:
-        """Internal utility to return all the primal-dual variables and indices that are
-        associated to non-redundant entries in the kkt conditions."""
-        if self.nlp.sym_type is cs.SX:
-            return self.nlp.primal_dual_vars(), slice(None)
-
-        # in case of MX, jacobians throw if the MX are indexed (no more
-        # symbolical according to the exception). So we take the jacobian
-        # with all primal-dual vars, and then index the relevant rows/cols.
-        y = self.nlp.primal_dual_vars(True)
-        h_lbx_idx = np.where(self.nlp.lbx != -np.inf)[0]
-        h_ubx_idx = np.where(self.nlp.ubx != +np.inf)[0]
-        n = self.nlp.nx + self.nlp.ng + self.nlp.nh
-        idx = np.concatenate(
-            (np.arange(n), h_lbx_idx + n, h_ubx_idx + n + h_lbx_idx.size)
-        )
-        return y, idx
-
-    @property
-    def _p_idx(self) -> Tuple[SymType, Union[slice, npt.NDArray[np.int64]]]:
-        """Internal utility to return the indices of p from all the NLP pars. Like for
-        _y_idx, SX is fine with jacobians, but MX jacobians need to be computed for
-        all elements and then indexed."""
+    def _p_idx(self) -> tuple[SymType, Union[slice, npt.NDArray[np.int64]]]:
+        """Internal utility to return the indices of p from all the NLP pars. While SX
+        is fine with computing jacobians with indexed variables, MX requires purely
+        symbolic variables. So, for MX, jacobians need to be computed for all elements
+        and then indexed."""
         p = self._p_idx_internal[0]
         return (self.nlp.p if p is None else p), self._p_idx_internal[1]
