@@ -102,8 +102,7 @@ class TestNonRetroactiveWrapper(unittest.TestCase):
 @parameterized_class("sym_type", [("SX",), ("MX",)])
 class TestNlpSensitivity(unittest.TestCase):
     def test_lagrangian__is_correct__example_1a_b(self):
-        # https://en.wikipedia.org/wiki/Lagrange_multiplier#Example_1a
-        # https://en.wikipedia.org/wiki/Lagrange_multiplier#Example_1b
+        # https://en.wikipedia.org/wiki/Lagrange_multiplier#Example_1
         nlp = NlpSensitivity(Nlp(sym_type=self.sym_type))
         x = nlp.variable("x")[0]
         y = nlp.variable("y")[0]
@@ -120,7 +119,7 @@ class TestNlpSensitivity(unittest.TestCase):
             )
 
     def test_lagrangian__is_correct__example_3(self):
-        # https://en.wikipedia.org/wiki/Lagrange_multiplier#Example_3:_Entropy
+        # https://en.wikipedia.org/wiki/Lagrange_multiplier#Example_3
         n = 50
         lbx = 1 / n * 1e-6
         nlp = NlpSensitivity(Nlp(sym_type=self.sym_type))
@@ -173,7 +172,7 @@ class TestNlpSensitivity(unittest.TestCase):
             self.assertIsNone(tau)
 
     def test_kkt__computes_kkt_conditions_correctly__example_1a(self):
-        # https://en.wikipedia.org/wiki/Lagrange_multiplier#Example_1a
+        # https://en.wikipedia.org/wiki/Lagrange_multiplier#Example_1
         nlp = NlpSensitivity(Nlp(sym_type=self.sym_type))
         x = nlp.variable("x")[0]
         y = nlp.variable("y")[0]
@@ -185,7 +184,7 @@ class TestNlpSensitivity(unittest.TestCase):
         np.testing.assert_allclose(sol.value(kkt), 0, atol=1e-9)
 
     def test_kkt__computes_kkt_conditions_correctly__example_1b(self):
-        # https://en.wikipedia.org/wiki/Lagrange_multiplier#Example_1b
+        # https://en.wikipedia.org/wiki/Lagrange_multiplier#Example_1
         nlp = NlpSensitivity(Nlp(sym_type=self.sym_type))
         x = nlp.variable("x")[0]
         y = nlp.variable("y")[0]
@@ -209,7 +208,7 @@ class TestNlpSensitivity(unittest.TestCase):
         np.testing.assert_allclose(sol.value(kkt), 0, atol=1e-6)
 
     def test_kkt__computes_kkt_conditions_correctly__example_3(self):
-        # https://en.wikipedia.org/wiki/Lagrange_multiplier#Example_3:_Entropy
+        # https://en.wikipedia.org/wiki/Lagrange_multiplier#Example_3
         n = 50
         nlp = NlpSensitivity(Nlp(sym_type=self.sym_type))
         p = nlp.variable("p", (n, 1), lb=1 / n * 1e-6)[0]
@@ -222,7 +221,7 @@ class TestNlpSensitivity(unittest.TestCase):
         np.testing.assert_allclose(sol.value(kkt), 0, atol=1e-6)
 
     def test_kkt__computes_kkt_conditions_correctly__example_4(self):
-        # https://en.wikipedia.org/wiki/Lagrange_multiplier#Example_4:_Numerical_optimization
+        # https://en.wikipedia.org/wiki/Lagrange_multiplier#Example_4
         nlp = NlpSensitivity(Nlp(sym_type=self.sym_type))
         x = nlp.variable("x")[0]
         nlp.constraint("c1", x**2, "==", 1)
@@ -438,208 +437,6 @@ class TestNlpSensitivity(unittest.TestCase):
         self.assertIn(repr(nlp), repr(nlp2))
 
 
-class TestMpc(unittest.TestCase):
-    def test_init__raises__with_invalid_args(self):
-        nlp = Nlp()
-        with self.assertRaisesRegex(ValueError, "Invalid shooting method."):
-            Mpc(nlp, shooting="ciao", prediction_horizon=1)
-        with self.assertRaisesRegex(
-            ValueError, "Prediction horizon must be positive and > 0."
-        ):
-            Mpc(nlp, prediction_horizon=0)
-        with self.assertRaisesRegex(
-            ValueError, "Control horizon must be positive and > 0."
-        ):
-            Mpc(nlp, prediction_horizon=10, control_horizon=-2)
-
-    def test_init__initializes_control_horizon_properly(self):
-        N = 10
-        nlp = Nlp()
-        mpc1 = Mpc[cs.SX](nlp, prediction_horizon=N)
-        mpc2 = Mpc[cs.SX](nlp, prediction_horizon=N, control_horizon=N * 2)
-        self.assertEqual(mpc1.control_horizon, N)
-        self.assertEqual(mpc2.control_horizon, N * 2)
-
-    @parameterized.expand([("single",), ("multi",)])
-    def test_state__constructs_state_correctly(self, shooting: str):
-        N = 10
-        nlp = Nlp(sym_type="MX")
-        mpc = Mpc[cs.MX](nlp=nlp, prediction_horizon=N, shooting=shooting)
-        x1, x1_0 = mpc.state("x1", 2)
-        if shooting == "multi":
-            self.assertEqual(x1.shape, (2, N + 1))
-            self.assertEqual(x1.shape, mpc.states["x1"].shape)
-            self.assertEqual(mpc.constraints["x1_0"].shape, (2, 1))
-        else:
-            self.assertIsNone(x1)
-        self.assertEqual(x1_0.shape, (2, 1))
-        self.assertEqual(x1_0.shape, mpc.initial_states["x1_0"].shape)
-        self.assertEqual(mpc.ns, x1_0.shape[0])
-        x2, x2_0 = mpc.state("x2", 1)
-        if shooting == "multi":
-            self.assertEqual(x2.shape, (1, N + 1))
-            self.assertEqual(x2.shape, mpc.states["x2"].shape)
-            self.assertEqual(mpc.constraints["x2_0"].shape, (1, 1))
-        else:
-            self.assertIsNone(x2)
-        self.assertEqual(x2_0.shape, (1, 1))
-        self.assertEqual(x2_0.shape, mpc.initial_states["x2_0"].shape)
-        self.assertEqual(mpc.ns, x1_0.shape[0] + x2_0.shape[0])
-
-    @parameterized.expand([(0,), (1,), (2,)])
-    def test_state__raises__in_singleshooting_with_state_bounds(self, i: int):
-        nlp = Nlp(sym_type="MX")
-        mpc = Mpc[cs.MX](nlp=nlp, prediction_horizon=10, shooting="single")
-        with self.assertRaises(RuntimeError):
-            if i == 0:
-                mpc.state("x1", 2, lb=0)
-            elif i == 1:
-                mpc.state("x1", 2, ub=1)
-            else:
-                mpc.state("x1", 2, lb=0, ub=1)
-
-    @parameterized.expand([(2,), (1,)])
-    def test_action__constructs_action_correctly(self, divider: int):
-        Np = 10
-        Nc = Np // divider
-        nlp = Nlp(sym_type="SX")
-        mpc = Mpc[cs.SX](nlp=nlp, prediction_horizon=Np, control_horizon=Nc)
-        u1, u1_exp = mpc.action("u1", 2)
-        self.assertEqual(u1.shape, (2, Nc))
-        self.assertEqual(u1.shape, mpc.actions["u1"].shape)
-        self.assertEqual(u1_exp.shape, (2, Np))
-        self.assertEqual(u1_exp.shape, mpc.actions_expanded["u1"].shape)
-        self.assertEqual(mpc.na, u1.shape[0])
-        u2, u2_exp = mpc.action("u2", 1)
-        self.assertEqual(u2.shape, (1, Nc))
-        self.assertEqual(u2.shape, mpc.actions["u2"].shape)
-        self.assertEqual(u2_exp.shape, (1, Np))
-        self.assertEqual(u2_exp.shape, mpc.actions_expanded["u2"].shape)
-        for i in range(Nc - 1, Np):
-            self.assertTrue(cs.is_equal(u1[:, -1], u1_exp[:, i]))
-            self.assertTrue(cs.is_equal(u2[:, -1], u2_exp[:, i]))
-        self.assertEqual(mpc.na, u1.shape[0] + u2.shape[0])
-
-    def test_constraint__constructs_slack_correctly(self):
-        nlp = Nlp(sym_type="MX")
-        mpc = Mpc[cs.MX](nlp=nlp, prediction_horizon=10)
-        x, _ = mpc.state("x1", 3)
-        _, _, slack = mpc.constraint("c0", x, ">=", 5, soft=True)
-        self.assertIn(slack.name(), mpc.slacks)
-        self.assertIn(slack.name(), mpc.slacks)
-        self.assertEqual(slack.shape, mpc.slacks[slack.name()].shape)
-        self.assertEqual(mpc.nslacks, x.shape[0])
-        mpc.constraint("c1", x, "<=", 10, soft=False)
-        self.assertEqual(mpc.nslacks, x.shape[0])
-
-    def test_disturbance__constructs_disturbance_correctly(self):
-        N = 10
-        nlp = Nlp(sym_type="MX")
-        mpc = Mpc[cs.MX](nlp=nlp, prediction_horizon=N)
-        d1 = mpc.disturbance("d1", 2)
-        self.assertEqual(d1.shape, (2, N))
-        self.assertEqual(d1.shape, mpc.disturbances["d1"].shape)
-        self.assertEqual(mpc.nd, 2)
-        d2 = mpc.disturbance("d2", 20)
-        self.assertEqual(d2.shape, (20, N))
-        self.assertEqual(d2.shape, mpc.disturbances["d2"].shape)
-        self.assertEqual(mpc.nd, 22)
-
-    def test_dynamics__raises__if_dynamics_already_set(self):
-        nlp = Nlp(sym_type="MX")
-        mpc = Mpc(nlp=nlp, prediction_horizon=10)
-        mpc._dynamics = 5
-        with self.assertRaises(RuntimeError):
-            mpc.set_dynamics(6)
-
-    def test_dynamics__raises__if_dynamics_arguments_are_invalid(self):
-        x1 = cs.SX.sym("x1", 2)
-        x2 = cs.SX.sym("x2", 3)
-        x = cs.vertcat(x1, x2)
-        u1 = cs.SX.sym("u1", 3)
-        u2 = cs.SX.sym("u2", 1)
-        u = cs.vertcat(u1, u2)
-        d = cs.SX.sym("d")
-        p = cs.SX.sym("p")
-        x_next = x + cs.vertcat(u, u2)
-        F1 = cs.Function("F", [x], [x_next])
-        F2 = cs.Function("F", [x, u, d, p], [x_next + d + p])
-        nlp = Nlp(sym_type="MX")
-        mpc = Mpc(nlp=nlp, prediction_horizon=10, control_horizon=5)
-        for F in (F1, F2):
-            with self.assertRaises(ValueError):
-                mpc.set_dynamics(F)
-
-    @parameterized.expand([(0,), (1,)])
-    def test_dynamics__in_multishooting__creates_dynamics_eq_constraints(self, i: int):
-        N = 10
-        nlp = Nlp(sym_type="SX")
-        mpc = Mpc[cs.SX](
-            nlp=nlp, prediction_horizon=N, control_horizon=N // 2, shooting="multi"
-        )
-        x1, _ = mpc.state("x1", 2)
-        x2, _ = mpc.state("x2", 3)
-        u1, _ = mpc.action("u1", 3)
-        u2, _ = mpc.action("u2", 1)
-        x = cs.vertcat(x1[:, 0], x2[:, 0])
-        u = cs.vertcat(u1[:, 0], u2[:, 0])
-        x_next = x + cs.vertcat(u, u2[:, 0])
-        if i == 0:
-            F = cs.Function("F", [x, u], [x_next], ["x", "u"], ["x+"])
-        else:
-            d = mpc.disturbance("d")
-            x_next += d[:, 0]
-            F = cs.Function("F", [x, u, d], [x_next], ["x", "u", "d"], ["x+"])
-        mpc.set_dynamics(F)
-        self.assertIn("dyn", mpc.constraints.keys())
-        self.assertEqual(mpc.ng, (1 + N) * 5)
-
-    @parameterized.expand([(0,), (1,)])
-    def test_dynamics__in_singleshooting__creates_states(self, i: int):
-        N = 10
-        nlp = Nlp(sym_type="SX")
-        mpc = Mpc[cs.SX](
-            nlp=nlp, prediction_horizon=N, control_horizon=N // 2, shooting="single"
-        )
-        mpc.state("x1", 2)
-        x1 = cs.SX.sym("x1", 2)
-        mpc.state("x2", 3)
-        x2 = cs.SX.sym("x2", 3)
-        u1, _ = mpc.action("u1", 3)
-        u2, _ = mpc.action("u2", 1)
-        x = cs.vertcat(x1[:, 0], x2[:, 0])
-        u = cs.vertcat(u1[:, 0], u2[:, 0])
-        x_next = x + cs.vertcat(u, u2[:, 0])
-        if i == 0:
-            F = cs.Function("F", [x, u], [x_next], ["x", "u"], ["x+"])
-        else:
-            d = mpc.disturbance("d")
-            x_next += d[:, 0]
-            F = cs.Function("F", [x, u, d], [x_next], ["x", "u", "d"], ["x+"])
-        mpc.set_dynamics(F)
-        for k in range(N):
-            self.assertNotIn(f"dyn_{k}", mpc.constraints.keys())
-        self.assertIn("x1", mpc.states.keys())
-        self.assertIn("x2", mpc.states.keys())
-        self.assertEqual(mpc.states["x1"].shape, (2, N + 1))
-        self.assertEqual(mpc.states["x2"].shape, (3, N + 1))
-
-    @parameterized.expand([("SX",), ("MX",)])
-    def test_can_be_pickled(self, sym_type: str):
-        N = 10
-        mpc = Mpc(
-            nlp=Nlp(sym_type=sym_type), prediction_horizon=N, control_horizon=N // 2
-        )
-        mpc.state("x1", 2)
-        mpc.state("x2", 3)
-        mpc.action("u1", 3)
-        mpc.action("u2", 1)
-
-        mpc2 = pickle.loads(pickle.dumps(mpc))
-
-        self.assertIn(repr(mpc), repr(mpc2))
-
-
 class TestNlpScaling(unittest.TestCase):
     @parameterized.expand([("variable",), ("parameter",)])
     def test_parameter_and_variable__warns__when_cannot_scale(self, method: str):
